Package: TestUseTry
Title: Test That `use_try` Parameter works
Version: 0.0.0.9000
Authors@R: "First Last <first.last@example.com> [aut, cre]"
Description: What the package does (one paragraph)
Depends:
    R (>= 3.1.2)
License: What license is it under?
LazyData: true
Suggests:
    testthat
