Package: TestS7
Title: What the Package Does (One Line, Title Case)
Version: 0.0.0.9000
Authors@R: c(
    person("Jim", "Hester", , "james.f.hester@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2739-7082")),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: What the package does (one paragraph).
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: S7
Suggests: 
    testthat (>= 3.0.0)
Config/testthat/edition: 3
