# S7 coverage is reported

    Code
      cov[, c("functions", "first_line", "last_line", "value")]
    Output
                                               functions first_line last_line value
      1                   Range@properties$length$getter          9         9     1
      2                   Range@properties$length$setter         11        11     1
      3                   Range@properties$length$setter         12        12     1
      4                                            Range         17        17     2
      5                                  Range@validator         20        20     5
      6                                  Range@validator         21        21     0
      7                                  Range@validator         22        22     5
      8                                  Range@validator         23        23     0
      9                                  Range@validator         24        24     5
      10                                 Range@validator         25        25     1
      11                   method(inside, TestS7::Range)         34        34     1
      12             method(base::format, TestS7::Range)         43        43     2
      13 method(testthat::testthat_print, TestS7::Range)         48        48     1
      14 method(testthat::testthat_print, TestS7::Range)         49        49     1
      15                                         .onLoad         53        53     0

