% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trace_calls.R
\name{key}
\alias{key}
\title{Generate a key for a  call}
\usage{
key(x)
}
\arguments{
\item{x}{the srcref of the call to create a key for}
}
\description{
Generate a key for a  call
}
\keyword{internal}
