% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trace_calls.R
\name{clear_counters}
\alias{clear_counters}
\title{clear all previous counters}
\usage{
clear_counters()
}
\description{
clear all previous counters
}
\keyword{internal}
