% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trace_calls.R
\name{count}
\alias{count}
\title{increment a given counter}
\usage{
count(key)
}
\arguments{
\item{key}{generated with \code{\link[=key]{key()}}}
}
\description{
increment a given counter
}
\keyword{internal}
