% Generated by roxygen2 (4.1.1): do not edit by hand
% Please edit documentation in R/TestExclusion.R
\name{test_me}
\alias{test_me}
\title{an example function}
\usage{
test_me(x, y)
}
\description{
an example function
}

