% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TestS4.R
\name{a}
\alias{a}
\title{an example function}
\usage{
a(x)
}
\description{
an example function
}
\examples{
a(1)
}

