% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TestCompiled.R
\name{simple}
\alias{simple}
\title{an example function}
\usage{
simple(x)
}
\description{
an example function
}
